#include <PSX/System/IOPorts/SPU_IO.hpp>
#include <PSX/System/IOPorts/IOPort.hpp>

namespace SPU {
    using namespace Port;

    static void clear_key() {
        Key::off.write(UI32_MAX);
    }

    static void clear_main_volume() {
        static constexpr auto StartVol = SweepVolume() | !SweepVolume::SweepEnable | (SweepVolume::Volume << static_cast<int16_t>(I16_MAX >> 2));

        MainVolume::left.write(StartVol);
        MainVolume::right.write(StartVol);
    }

    static void clear_control_register() {
        Control.write(ControlRegister());
    }

    static void clear_voice() {
        for(auto& voice : Voices) {
            voice.volumeLeft.write(SweepVolume());
            voice.volumeRight.write(SweepVolume());
            voice.sampleRate.write(SampleRate());
            voice.ad.write(AD());
            voice.sr.write(SR());
            voice.currentVolume.write(SweepVolume());
            
            voice.adr.write(0x200);
            voice.repeatAdr.write(0x200);
        }
    }

    static void setup_control_register() {
        static constexpr auto SetupValue = ControlRegister() | ControlRegister::Enable | ControlRegister::Unmute | ControlRegister::CDAudioEnable;

        Control.write(SetupValue);
    }

    void setup() {
        clear_key();
        clear_main_volume();
        clear_control_register();
        clear_voice();

        setup_control_register();
    }
}